@@ -1,4 +1,4 @@
-# Copyright (C) 2014-2016 MongoDB, Inc.
+# Copyright (C) 2014-2017 MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the 'License');
 # you may not use this file except in compliance with the License.
@@ -38,11 +38,13 @@ module Mongo
       :auth_source,
       :connect,
       :connect_timeout,
+      :compressors,
       :database,
       :heartbeat_frequency,
       :id_generator,
       :local_threshold,
       :logger,
+      :max_idle_time,
       :max_pool_size,
       :max_read_retries,
       :min_pool_size,
@@ -52,6 +54,7 @@ module Mongo
       :read,
       :read_retry_interval,
       :replica_set,
+      :retry_writes,
       :server_selection_timeout,
       :socket_timeout,
       :ssl,
@@ -62,16 +65,22 @@ module Mongo
       :ssl_cert_string,
       :ssl_cert_object,
       :ssl_key,
-      :ssk_key_string,
+      :ssl_key_string,
       :ssl_key_object,
       :ssl_key_pass_phrase,
       :ssl_verify,
       :truncate_logs,
       :user,
       :wait_queue_timeout,
-      :write
+      :write,
+      :zlib_compression_level
     ].freeze
 
+    # The compression algorithms supported by the driver.
+    #
+    # @since 2.5.0
+    VALID_COMPRESSORS = [ Mongo::Protocol::Compressed::ZLIB ].freeze
+
     # @return [ Mongo::Cluster ] cluster The cluster of servers for the client.
     attr_reader :cluster
 
@@ -157,6 +166,8 @@ module Mongo
     # @option options [ Integer ] :server_selection_timeout The timeout in seconds
     #   for selecting a server for an operation.
     # @option options [ String ] :password The user's password.
+    # @option options [ Integer ] :max_idle_time The maximum seconds a socket can remain idle
+    #   since it has been checked in to the pool.
     # @option options [ Integer ] :max_pool_size The maximum size of the
     #   connection pool.
     # @option options [ Integer ] :min_pool_size The minimum size of the
@@ -165,6 +176,9 @@ module Mongo
     #   seconds, in the connection pool for a connection to be checked in.
     # @option options [ Float ] :connect_timeout The timeout, in seconds, to
     #   attempt a connection.
+    # @option options [ Array<String> ] :compressors A list of potential compressors to use, in order of preference.
+    #  The driver chooses the first compressor that is also supported by the server. Currently the driver only
+    #   supports 'zlib'.
     # @option options [ Hash ] :read The read preference options. They consist of a
     #   mode specified as a symbol, an array of hashes known as tag_sets,
     #   and local_threshold.
@@ -225,14 +239,18 @@ module Mongo
     #   mongod logs upon establishing a connection in server versions >= 3.4.
     # @option options [ String ] :platform Platform information to include in the
     #   metadata printed to the mongod logs upon establishing a connection in server versions >= 3.4.
+    # @option options [ Integer ] :zlib_compression_level The Zlib compression level to use, if using compression.
+    #   See Ruby's Zlib module for valid levels.
+    # @option options [ true, false ] :retry_writes Retry writes once when connected to a replica set
+    #   or sharded cluster versions 3.6 and up.
     #
     # @since 2.0.0
     def initialize(addresses_or_uri, options = Options::Redacted.new)
       @monitoring = Monitoring.new(options)
       if addresses_or_uri.is_a?(::String)
-        create_from_uri(addresses_or_uri, validate_options(options))
+        create_from_uri(addresses_or_uri, validate_options!(options))
       else
-        create_from_addresses(addresses_or_uri, validate_options(options))
+        create_from_addresses(addresses_or_uri, validate_options!(options))
       end
       yield(self) if block_given?
     end
@@ -249,17 +267,30 @@ module Mongo
       "#<Mongo::Client:0x#{object_id} cluster=#{cluster.addresses.join(', ')}>"
     end
 
+    # Get the server selector. It either uses the read preference defined in the client options
+    #   or defaults to a Primary server selector.
+    #
+    # @example Get the server selector.
+    #   client.server_selector
+    #
+    # @return [ Mongo::ServerSelector ] The server selector using the user-defined read preference
+    #  or a Primary server selector default.
+    #
+    # @since 2.5.0
+    def server_selector
+      @server_selector ||= ServerSelector.get(read_preference || ServerSelector::PRIMARY)
+    end
+
     # Get the read preference from the options passed to the client.
     #
     # @example Get the read preference.
     #   client.read_preference
     #
-    # @return [ Object ] The appropriate read preference or primary if none
-    #   was provided to the client.
+    # @return [ BSON::Document ] The user-defined read preference.
     #
     # @since 2.0.0
     def read_preference
-      @read_preference ||= ServerSelector.get(options[:read] || ServerSelector::PRIMARY)
+      @read_preference ||= options[:read]
     end
 
     # Use the database with the provided name. This will switch the current
@@ -291,7 +322,7 @@ module Mongo
     # @since 2.0.0
     def with(new_options = Options::Redacted.new)
       clone.tap do |client|
-        opts = validate_options(new_options)
+        opts = validate_options!(new_options)
         client.options.update(opts)
         Database.create(client)
         # We can't use the same cluster if some options that would affect it
@@ -344,11 +375,14 @@ module Mongo
     # @example Get the database names.
     #   client.database_names
     #
+    # @param [ Hash ] filter The filter criteria for getting a list of databases.
+    # @param [ Hash ] opts The command options.
+    #
     # @return [ Array<String> ] The names of the databases.
     #
     # @since 2.0.5
-    def database_names
-      list_databases.collect{ |info| info[Database::NAME] }
+    def database_names(filter = {}, opts = {})
+      list_databases(filter, true, opts).collect{ |info| info[Database::NAME] }
     end
 
     # Get info for each database.
@@ -356,15 +390,65 @@ module Mongo
     # @example Get the info for each database.
     #   client.list_databases
     #
+    # @param [ Hash ] filter The filter criteria for getting a list of databases.
+    # @param [ true, false ] name_only Whether to only return each database name without full metadata.
+    # @param [ Hash ] opts The command options.
+    #
     # @return [ Array<Hash> ] The info for each database.
     #
     # @since 2.0.5
-    def list_databases
-      use(Database::ADMIN).command(listDatabases: 1).first[Database::DATABASES]
+    def list_databases(filter = {}, name_only = false, opts = {})
+      cmd = { listDatabases: 1 }
+      cmd[:nameOnly] = !!name_only
+      cmd[:filter] = filter unless filter.empty?
+      use(Database::ADMIN).command(cmd, opts).first[Database::DATABASES]
+    end
+
+    # Returns a list of Mongo::Database objects.
+    #
+    # @example Get a list of Mongo::Database objects.
+    #   client.list_mongo_databases
+    #
+    # @param [ Hash ] filter The filter criteria for getting a list of databases.
+    # @param [ Hash ] opts The command options.
+    #
+    # @return [ Array<Mongo::Database> ] The list of database objects.
+    #
+    # @since 2.5.0
+    def list_mongo_databases(filter = {}, opts = {})
+      database_names(filter, opts).collect do |name|
+        Database.new(self, name, options)
+      end
+    end
+
+    # Start a session.
+    #
+    # @example Start a session.
+    #   client.start_session(causal_consistency: true)
+    #
+    # @param [ Hash ] options The session options.
+    #
+    # @note A Session cannot be used by multiple threads at once; session objects are not
+    #   thread-safe.
+    #
+    # @return [ Session ] The session.
+    #
+    # @since 2.5.0
+    def start_session(options = {})
+      cluster.send(:get_session, options.merge(implicit: false)) ||
+        (raise Error::InvalidSession.new(Session::SESSIONS_NOT_SUPPORTED))
     end
 
     private
 
+    def get_session(options = {})
+      cluster.send(:get_session, options)
+    end
+
+    def with_session(options = {}, &block)
+      cluster.send(:with_session, options, &block)
+    end
+
     def create_from_addresses(addresses, opts = Options::Redacted.new)
       @options = Database::DEFAULT_OPTIONS.merge(opts).freeze
       @cluster = Cluster.new(addresses, @monitoring, options)
@@ -372,8 +456,8 @@ module Mongo
     end
 
     def create_from_uri(connection_string, opts = Options::Redacted.new)
-      uri = URI.new(connection_string, opts)
-      @options = Database::DEFAULT_OPTIONS.merge(uri.client_options.merge(opts)).freeze
+      uri = URI.get(connection_string, opts)
+      @options = validate_options!(Database::DEFAULT_OPTIONS.merge(uri.client_options.merge(opts))).freeze
       @cluster = Cluster.new(uri.servers, @monitoring, options)
       @database = Database.new(self, options[:database], options)
     end
@@ -395,16 +479,43 @@ module Mongo
       end
     end
 
-    def validate_options(opts = Options::Redacted.new)
+    def validate_options!(opts = Options::Redacted.new)
       return Options::Redacted.new unless opts
-      Options::Redacted.new(opts.select do |o|
-        if VALID_OPTIONS.include?(o)
-          true
+      opts.each.inject(Options::Redacted.new) do |_options, (k, v)|
+        key = k.to_sym
+        if VALID_OPTIONS.include?(key)
+          validate_max_min_pool_size!(key, opts)
+          if key == :compressors
+            compressors = valid_compressors(v)
+            _options[key] = compressors unless compressors.empty?
+          else
+            _options[key] = v
+          end
         else
-          log_warn("Unsupported client option '#{o}'. It will be ignored.")
-          false
+          log_warn("Unsupported client option '#{k}'. It will be ignored.")
         end
-      end)
+        _options
+      end
+    end
+
+    def valid_compressors(compressors)
+      compressors.select do |compressor|
+        if !VALID_COMPRESSORS.include?(compressor)
+          log_warn("Unsupported compressor '#{compressor}' in list '#{compressors}'. " +
+                       "This compressor will not be used.")
+          false
+        else
+          true
+        end
+      end
+    end
+
+    def validate_max_min_pool_size!(option, opts)
+      if option == :min_pool_size && opts[:min_pool_size]
+        max = opts[:max_pool_size] || Server::ConnectionPool::Queue::MAX_SIZE
+        raise Error::InvalidMinPoolSize.new(opts[:min_pool_size], max) unless opts[:min_pool_size] <= max
+      end
+      true
     end
   end
 end
