@@ -1,4 +1,4 @@
-# Copyright (C) 2014-2016 MongoDB, Inc.
+# Copyright (C) 2014-2017 MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -50,9 +50,6 @@ module Mongo
     # Delegate to the cluster for the next primary.
     def_delegators :cluster, :next_primary
 
-    # Convenience delegators to find.
-    def_delegators :find, :parallel_scan
-
     # Options that can be updated on a new Collection instance via the #with method.
     #
     # @since 2.1.0
@@ -103,16 +100,28 @@ module Mongo
       @read_concern ||= options[:read_concern]
     end
 
+    # Get the server selector on this collection.
+    #
+    # @example Get the server selector.
+    #   collection.server_selector
+    #
+    # @return [ Mongo::ServerSelector ] The server selector.
+    #
+    # @since 2.0.0
+    def server_selector
+     @server_selector ||= ServerSelector.get(read_preference || database.server_selector)
+    end
+
     # Get the read preference on this collection.
     #
     # @example Get the read preference.
     #   collection.read_preference
     #
-    # @return [ Mongo::ServerSelector ] The read preference.
+    # @return [ Hash ] The read preference.
     #
     # @since 2.0.0
     def read_preference
-      @read_preference ||= ServerSelector.get(options[:read] || database.read_preference)
+      @read_preference ||= options[:read] || database.read_preference
     end
 
     # Get the write concern on this collection.
@@ -165,21 +174,28 @@ module Mongo
     # @example Force the collection to be created.
     #   collection.create
     #
+    # @param [ Hash ] opts The options for the create operation.
+    #
+    # @option options [ Session ] :session The session to use for the operation.
+    #
     # @return [ Result ] The result of the command.
     #
     # @since 2.0.0
-    def create
+    def create(opts = {})
       operation = { :create => name }.merge(options)
       operation.delete(:write)
       server = next_primary
       if (options[:collation] || options[Operation::COLLATION]) && !server.features.collation_enabled?
         raise Error::UnsupportedCollation.new
       end
-      Operation::Commands::Create.new({
-                                        selector: operation,
-                                        db_name: database.name,
-                                        write_concern: write_concern
-                                      }).execute(server)
+      client.send(:with_session, opts) do |session|
+        Operation::Commands::Create.new({
+                                          selector: operation,
+                                          db_name: database.name,
+                                          write_concern: write_concern,
+                                          session: session
+                                        }).execute(server)
+      end
     end
 
     # Drop the collection. Will also drop all indexes associated with the
@@ -190,16 +206,22 @@ module Mongo
     # @example Drop the collection.
     #   collection.drop
     #
+    # @param [ Hash ] opts The options for the drop operation.
+    #
+    # @option options [ Session ] :session The session to use for the operation.
+    #
     # @return [ Result ] The result of the command.
     #
     # @since 2.0.0
-    def drop
-      Operation::Commands::Drop.new({
-                                      selector: { :drop => name },
-                                      db_name: database.name,
-                                      write_concern: write_concern
-                                    }).execute(next_primary)
-
+    def drop(opts = {})
+      client.send(:with_session, opts) do |session|
+        Operation::Commands::Drop.new({
+                                        selector: { :drop => name },
+                                        db_name: database.name,
+                                        write_concern: write_concern,
+                                        session: session
+                                      }).execute(next_primary)
+      end
     rescue Error::OperationFailure => ex
       raise ex unless ex.message =~ /ns not found/
       false
@@ -238,6 +260,7 @@ module Mongo
     # @option options [ Hash ] :sort The key and direction pairs by which the result set
     #   will be sorted.
     # @option options [ Hash ] :collation The collation to use.
+    # @option options [ Session ] :session The session to use.
     #
     # @return [ CollectionView ] The collection view.
     #
@@ -260,10 +283,13 @@ module Mongo
     # @option options [ Integer ] :max_time_ms The maximum amount of time in milliseconds to allow the
     #   aggregation to run.
     # @option options [ true, false ] :use_cursor Indicates whether the command will request that the server
-    #   provide results using a cursor.
+    #   provide results using a cursor. Note that as of server version 3.6, aggregations always provide results
+    #   using a cursor and this option is therefore not valid.
     # @option options [ true, false ] :bypass_document_validation Whether or
     #   not to skip document level validation.
     # @option options [ Hash ] :collation The collation to use.
+    # @option options [ String ] :comment Associate a comment with the aggregation.
+    # @option options [ Session ] :session The session to use.
     #
     # @return [ Aggregation ] The aggregation object.
     #
@@ -272,6 +298,39 @@ module Mongo
       View.new(self, {}).aggregate(pipeline, options)
     end
 
+    # As of version 3.6 of the MongoDB server, a ``$changeStream`` pipeline stage is supported
+    # in the aggregation framework. This stage allows users to request that notifications are sent for
+    # all changes to a particular collection.
+    #
+    # @example Get change notifications for a given collection.
+    #   collection.watch([{ '$match' => { operationType: { '$in' => ['insert', 'replace'] } } }])
+    #
+    # @param [ Array<Hash> ] pipeline Optional additional filter operators.
+    # @param [ Hash ] options The change stream options.
+    #
+    # @option options [ String ] :full_document Allowed values: ‘default’, ‘updateLookup’. Defaults to ‘default’.
+    #   When set to ‘updateLookup’, the change notification for partial updates will include both a delta
+    #   describing the changes to the document, as well as a copy of the entire document that was changed
+    #   from some time after the change occurred.
+    # @option options [ BSON::Document, Hash ] :resume_after Specifies the logical starting point for the
+    #   new change stream.
+    # @option options [ Integer ] :max_await_time_ms The maximum amount of time for the server to wait
+    #   on new documents to satisfy a change stream query.
+    # @option options [ Integer ] :batch_size The number of documents to return per batch.
+    # @option options [ BSON::Document, Hash ] :collation The collation to use.
+    # @option options [ Session ] :session The session to use.
+    #
+    # @note A change stream only allows 'majority' read concern.
+    # @note This helper method is preferable to running a raw aggregation with a $changeStream stage,
+    #   for the purpose of supporting resumability.
+    #
+    # @return [ ChangeStream ] The change stream object.
+    #
+    # @since 2.5.0
+    def watch(pipeline = [], options = {})
+      View::ChangeStream.new(View.new(self, {}, options), pipeline, options)
+    end
+
     # Get a count of matching documents in the collection.
     #
     # @example Get the count.
@@ -286,6 +345,7 @@ module Mongo
     # @option options [ Integer ] :skip The number of documents to skip before counting.
     # @option options [ Hash ] :read The read preference options.
     # @option options [ Hash ] :collation The collation to use.
+    # @option options [ Session ] :session The session to use.
     #
     # @return [ Integer ] The document count.
     #
@@ -306,6 +366,7 @@ module Mongo
     # @option options [ Integer ] :max_time_ms The maximum amount of time to allow the command to run.
     # @option options [ Hash ] :read The read preference options.
     # @option options [ Hash ] :collation The collation to use.
+    # @option options [ Session ] :session The session to use.
     #
     # @return [ Array<Object> ] The list of distinct values.
     #
@@ -322,6 +383,8 @@ module Mongo
     #
     # @param [ Hash ] options Options for getting a list of all indexes.
     #
+    # @option options [ Session ] :session The session to use.
+    #
     # @return [ View::Index ] The index view.
     #
     # @since 2.0.0
@@ -347,22 +410,28 @@ module Mongo
     #   collection.insert_one({ name: 'test' })
     #
     # @param [ Hash ] document The document to insert.
-    # @param [ Hash ] options The insert options.
+    # @param [ Hash ] opts The insert options.
+    #
+    # @option opts [ Session ] :session The session to use for the operation.
     #
     # @return [ Result ] The database response wrapper.
     #
     # @since 2.0.0
-    def insert_one(document, options = {})
-      write_with_retry do
-        Operation::Write::Insert.new(
-          :documents => [ document ],
-          :db_name => database.name,
-          :coll_name => name,
-          :write_concern => write_concern,
-          :bypass_document_validation => !!options[:bypass_document_validation],
-          :options => options,
-          :id_generator => client.options[:id_generator]
-        ).execute(next_primary)
+    def insert_one(document, opts = {})
+      client.send(:with_session, opts) do |session|
+        write_with_retry(session, write_concern) do |server, txn_num|
+          Operation::Write::Insert.new(
+              :documents => [ document ],
+              :db_name => database.name,
+              :coll_name => name,
+              :write_concern => write_concern,
+              :bypass_document_validation => !!opts[:bypass_document_validation],
+              :options => opts,
+              :id_generator => client.options[:id_generator],
+              :session => session,
+              :txn_num => txn_num
+           ).execute(server)
+        end
       end
     end
 
@@ -374,6 +443,8 @@ module Mongo
     # @param [ Array<Hash> ] documents The documents to insert.
     # @param [ Hash ] options The insert options.
     #
+    # @option options [ Session ] :session The session to use for the operation.
+    #
     # @return [ Result ] The database response wrapper.
     #
     # @since 2.0.0
@@ -396,6 +467,7 @@ module Mongo
     #   Can be :w => Integer, :fsync => Boolean, :j => Boolean.
     # @option options [ true, false ] :bypass_document_validation Whether or
     #   not to skip document level validation.
+    # @option options [ Session ] :session The session to use for the set of operations.
     #
     # @return [ BulkWrite::Result ] The result of the operation.
     #
@@ -413,6 +485,7 @@ module Mongo
     # @param [ Hash ] options The options.
     #
     # @option options [ Hash ] :collation The collation to use.
+    # @option options [ Session ] :session The session to use.
     #
     # @return [ Result ] The response from the database.
     #
@@ -430,6 +503,7 @@ module Mongo
     # @param [ Hash ] options The options.
     #
     # @option options [ Hash ] :collation The collation to use.
+    # @option options [ Session ] :session The session to use.
     #
     # @return [ Result ] The response from the database.
     #
@@ -452,12 +526,13 @@ module Mongo
     #
     # @option options [ Integer ] :max_time_ms The maximum amount of time to allow the command
     #   to run in milliseconds.
+    # @option options [ Session ] :session The session to use.
     #
     # @return [ Array<Cursor> ] An array of cursors.
     #
     # @since 2.1
     def parallel_scan(cursor_count, options = {})
-      find.send(:parallel_scan, cursor_count, options)
+      find({}, options).send(:parallel_scan, cursor_count, options)
     end
 
     # Replaces a single document in the collection with the new document.
@@ -474,6 +549,7 @@ module Mongo
     # @option options [ true, false ] :bypass_document_validation Whether or
     #   not to skip document level validation.
     # @option options [ Hash ] :collation The collation to use.
+    # @option options [ Session ] :session The session to use.
     #
     # @return [ Result ] The response from the database.
     #
@@ -496,6 +572,9 @@ module Mongo
     # @option options [ true, false ] :bypass_document_validation Whether or
     #   not to skip document level validation.
     # @option options [ Hash ] :collation The collation to use.
+    # @option options [ Array ] :array_filters A set of filters specifying to which array elements
+    #   an update should apply.
+    # @option options [ Session ] :session The session to use.
     #
     # @return [ Result ] The response from the database.
     #
@@ -518,6 +597,9 @@ module Mongo
     # @option options [ true, false ] :bypass_document_validation Whether or
     #   not to skip document level validation.
     # @option options [ Hash ] :collation The collation to use.
+    # @option options [ Array ] :array_filters A set of filters specifying to which array elements
+    #   an update should apply.
+    # @option options [ Session ] :session The session to use.
     #
     # @return [ Result ] The response from the database.
     #
@@ -543,12 +625,13 @@ module Mongo
     # @option options [ Hash ] :write_concern The write concern options.
     #   Defaults to the collection's write concern.
     # @option options [ Hash ] :collation The collation to use.
+    # @option options [ Session ] :session The session to use.
     #
     # @return [ BSON::Document, nil ] The document, if found.
     #
     # @since 2.1.0
     def find_one_and_delete(filter, options = {})
-      find(filter, options).find_one_and_delete
+      find(filter, options).find_one_and_delete(options)
     end
 
     # Finds a single document via findAndModify and updates it, returning the original doc unless
@@ -576,6 +659,9 @@ module Mongo
     # @option options [ Hash ] :write_concern The write concern options.
     #   Defaults to the collection's write concern.
     # @option options [ Hash ] :collation The collation to use.
+    # @option options [ Array ] :array_filters A set of filters specifying to which array elements
+    #   an update should apply.
+    # @option options [ Session ] :session The session to use.
     #
     # @return [ BSON::Document ] The document.
     #
@@ -609,6 +695,7 @@ module Mongo
     # @option options [ Hash ] :write_concern The write concern options.
     #   Defaults to the collection's write concern.
     # @option options [ Hash ] :collation The collation to use.
+    # @option options [ Session ] :session The session to use.
     #
     # @return [ BSON::Document ] The document.
     #
