@@ -1,4 +1,4 @@
-# Copyright (C) 2014-2016 MongoDB, Inc.
+# Copyright (C) 2014-2017 MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,6 +14,9 @@
 
 require 'mongo/operation/commands/command'
 require 'mongo/operation/commands/find'
+require 'mongo/operation/commands/count'
+require 'mongo/operation/commands/distinct'
+require 'mongo/operation/commands/explain'
 require 'mongo/operation/commands/get_more'
 require 'mongo/operation/commands/parallel_scan'
 require 'mongo/operation/commands/aggregate'
@@ -25,5 +28,4 @@ require 'mongo/operation/commands/drop_database'
 require 'mongo/operation/commands/indexes'
 require 'mongo/operation/commands/list_collections'
 require 'mongo/operation/commands/list_indexes'
-require 'mongo/operation/commands/user_query'
 require 'mongo/operation/commands/users_info'
