@@ -1,4 +1,4 @@
-# Copyright (C) 2014-2016 MongoDB, Inc.
+# Copyright (C) 2014-2017 MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,7 +19,7 @@ module Mongo
       # A MongoDB command operation.
       #
       # @example Create the command operation.
-      #   Mongo::Operation::Command.new({ :selector => { :isMaster => 1 } })
+      #   Command.new({ :selector => { :isMaster => 1 } })
       #
       # Initialization:
       #   param [ Hash ] spec The specifications for the command.
@@ -35,6 +35,7 @@ module Mongo
         include Limited
         include ReadPreference
         include Executable
+        include UsesCommandOpMsg
 
         private
 
