@@ -1,5 +1,5 @@
 
-# Copyright (C) 2014-2016 MongoDB, Inc.
+# Copyright (C) 2014-2017 MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -25,6 +25,25 @@ module Mongo
           include Specifiable
           include Writable
 
+          # Execute the operation.
+          #
+          # @example Execute the operation.
+          #   operation.execute(server)
+          #
+          # @param [ Mongo::Server ] server The server to send this operation to.
+          #
+          # @return [ Result ] The operation response, if there is one.
+          #
+          # @since 2.5.0
+          def execute(server)
+            result = Result.new(server.with_connection do |connection|
+              connection.dispatch([ message(server) ], operation_id)
+            end)
+            server.update_cluster_time(result)
+            session.process(result) if session
+            result.validate!
+          end
+
           private
 
           # The query selector for this create user command operation.
