@@ -1,4 +1,4 @@
-# Copyright (C) 2014-2016 MongoDB, Inc.
+# Copyright (C) 2014-2017 MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the 'License');
 # you may not use this file except in compliance with the License.
@@ -124,6 +124,11 @@ module Mongo
       # @since 2.0.0
       PRIMARY = 'ismaster'.freeze
 
+      # Constant for reading primary host field from config.
+      #
+      # @since 2.5.0
+      PRIMARY_HOST = 'primary'.freeze
+
       # Constant for reading secondary info from config.
       #
       # @since 2.0.0
@@ -154,10 +159,23 @@ module Mongo
       # @since 2.1.0
       LOCAL_TIME = 'localTime'.freeze
 
+      # Constant for reading operationTime info from config.
+      #
+      # @since 2.5.0
+      OPERATION_TIME = 'operationTime'.freeze
+
+      # Constant for reading logicalSessionTimeoutMinutes info from config.
+      #
+      # @since 2.5.0
+      LOGICAL_SESSION_TIMEOUT_MINUTES = 'logicalSessionTimeoutMinutes'.freeze
+
       # Fields to exclude when comparing two descriptions.
       #
       # @since 2.0.6
-      EXCLUDE_FOR_COMPARISON = [ LOCAL_TIME ].freeze
+      EXCLUDE_FOR_COMPARISON = [ LOCAL_TIME,
+                                 LAST_WRITE,
+                                 OPERATION_TIME,
+                                 Operation::CLUSTER_TIME ].freeze
 
       # @return [ Address ] address The server's address.
       attr_reader :address
@@ -246,7 +264,7 @@ module Mongo
       def initialize(address, config = {}, average_round_trip_time = 0)
         @address = address
         @config = config
-        @features = Features.new(wire_versions)
+        @features = Features.new(wire_versions, me || @address.to_s)
         @average_round_trip_time = average_round_trip_time
       end
 
@@ -382,6 +400,18 @@ module Mongo
         config[LAST_WRITE][LAST_WRITE_DATE] if config[LAST_WRITE]
       end
 
+      # Get the logicalSessionTimeoutMinutes from the config.
+      #
+      # @example Get the logicalSessionTimeoutMinutes value in minutes.
+      #   description.logical_session_timeout
+      #
+      # @return [ Integer, nil ] The logical session timeout in minutes.
+      #
+      # @since 2.5.0
+      def logical_session_timeout
+        config[LOGICAL_SESSION_TIMEOUT_MINUTES] if config[LOGICAL_SESSION_TIMEOUT_MINUTES]
+      end
+
       # Is the server a mongos?
       #
       # @example Is the server a mongos?
@@ -440,7 +470,9 @@ module Mongo
       #
       # @since 2.0.0
       def primary?
-        !!config[PRIMARY] && !replica_set_name.nil?
+        !!config[PRIMARY] &&
+          (config[PRIMARY_HOST].nil? || config[PRIMARY_HOST] == address.to_s) &&
+            !replica_set_name.nil?
       end
 
       # Get the name of the replica set the server belongs to, returns nil if
