@@ -1,4 +1,4 @@
-# Copyright (C) 2014-2016 MongoDB, Inc.
+# Copyright (C) 2014-2017 MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the 'License');
 # you may not use this file except in compliance with the License.
@@ -22,8 +22,8 @@ module Mongo
   # http://docs.mongodb.org/manual/reference/connection-string/
   #
   # @example Use the uri string to make a client connection.
-  #   uri = URI.new('mongodb://localhost:27017')
-  #   client = Client.new(uri.server, uri.options)
+  #   uri = Mongo::URI.new('mongodb://localhost:27017')
+  #   client = Mongo::Client.new(uri.servers, uri.options)
   #   client.login(uri.credentials)
   #   client[uri.database]
   #
@@ -46,21 +46,54 @@ module Mongo
     # @since 2.0.0
     attr_reader :servers
 
+    # The mongodb connection string scheme.
+    #
+    # @deprecated Will be removed in 3.0.
+    #
+    # @since 2.0.0
+    SCHEME = 'mongodb://'.freeze
+
+    # The mongodb connection string scheme root.
+    #
+    # @since 2.5.0
+    MONGODB_SCHEME = 'mongodb'.freeze
+
+    # The mongodb srv protocol connection string scheme root.
+    #
+    # @since 2.5.0
+    MONGODB_SRV_SCHEME = 'mongodb+srv'.freeze
+
+    # Error details for an invalid scheme.
+    #
+    # @since 2.1.0
+    INVALID_SCHEME = "Invalid scheme. Scheme must be '#{MONGODB_SCHEME}' or '#{MONGODB_SRV_SCHEME}'".freeze
+
+    # MongoDB URI format specification.
+    #
+    # @since 2.0.0
+    FORMAT = 'mongodb://[username:password@]host1[:port1][,host2[:port2]' +
+        ',...[,hostN[:portN]]][/[database][?options]]'.freeze
+
+    # MongoDB URI (connection string) documentation url
+    #
+    # @since 2.0.0
+    HELP = 'http://docs.mongodb.org/manual/reference/connection-string/'.freeze
+
     # Unsafe characters that must be urlencoded.
     #
     # @since 2.1.0
     UNSAFE = /[\:\/\+\@]/
 
+    # Percent sign that must be encoded in user creds.
+    #
+    # @since 2.5.1
+    PERCENT_CHAR = /\%/
+
     # Unix socket suffix.
     #
     # @since 2.1.0
     UNIX_SOCKET = /.sock/
 
-    # The mongodb connection string scheme.
-    #
-    # @since 2.0.0
-    SCHEME = 'mongodb://'.freeze
-
     # The character delimiting hosts.
     #
     # @since 2.1.0
@@ -101,10 +134,10 @@ module Mongo
     # @since 2.1.0
     AUTH_DELIM = '@'.freeze
 
-    # Error details for an invalid scheme.
+    # Scheme delimiter.
     #
-    # @since 2.1.0
-    INVALID_SCHEME = "Invalid scheme. Scheme must be '#{SCHEME}'".freeze
+    # @since 2.5.0
+    SCHEME_DELIM = '://'.freeze
 
     # Error details for an invalid options format.
     #
@@ -142,17 +175,6 @@ module Mongo
     # @since 2.1.0
     INVALID_PORT = "Invalid port. Port must be an integer greater than 0 and less than 65536".freeze
 
-    # MongoDB URI format specification.
-    #
-    # @since 2.0.0
-    FORMAT = 'mongodb://[username:password@]host1[:port1][,host2[:port2]' +
-      ',...[,hostN[:portN]]][/[database][?options]]'.freeze
-
-    # MongoDB URI (connection string) documentation url
-    #
-    # @since 2.0.0
-    HELP = 'http://docs.mongodb.org/manual/reference/connection-string/'.freeze
-
     # Map of URI read preference modes to ruby driver read preference modes
     #
     # @since 2.0.0
@@ -180,23 +202,24 @@ module Mongo
     # @since 2.1.0
     REPEATABLE_OPTIONS = [ :tag_sets ]
 
-    # Create the new uri from the provided string.
+    # Get either a URI object or a SRVProtocol URI object.
     #
-    # @example Create the new URI.
-    #   URI.new('mongodb://localhost:27017')
+    # @example Get the uri object.
+    #   URI.get(string)
     #
-    # @param [ String ] string The uri string.
-    # @param [ Hash ] options The options.
+    # @return [URI, URI::SRVProtocol] The uri object.
     #
-    # @raise [ Error::InvalidURI ] If the uri does not match the spec.
-    #
-    # @since 2.0.0
-    def initialize(string, options = {})
-      @string = string
-      @options = options
-      _, scheme, remaining = @string.partition(SCHEME)
-      raise_invalid_error!(INVALID_SCHEME) unless scheme == SCHEME
-      setup!(remaining)
+    # @since 2.5.0
+    def self.get(string, opts = {})
+      scheme, _, remaining = string.partition(SCHEME_DELIM)
+      case scheme
+        when MONGODB_SCHEME
+          URI.new(string, opts)
+        when MONGODB_SRV_SCHEME
+          SRVProtocol.new(string, opts)
+        else
+          raise Error::InvalidURI.new(string, INVALID_SCHEME)
+      end
     end
 
     # Gets the options hash that needs to be passed to a Mongo::Client on
@@ -214,6 +237,25 @@ module Mongo
       @user ? opts.merge(credentials) : opts
     end
 
+    # Create the new uri from the provided string.
+    #
+    # @example Create the new URI.
+    #   URI.new('mongodb://localhost:27017')
+    #
+    # @param [ String ] string The uri string.
+    # @param [ Hash ] options The options.
+    #
+    # @raise [ Error::InvalidURI ] If the uri does not match the spec.
+    #
+    # @since 2.0.0
+    def initialize(string, options = {})
+      @string = string
+      @options = options
+      parsed_scheme, _, remaining = string.partition(SCHEME_DELIM)
+      raise_invalid_error!(INVALID_SCHEME) unless parsed_scheme == scheme
+      parse!(remaining)
+    end
+
     # Get the credentials provided in the URI.
     #
     # @example Get the credentials.
@@ -242,7 +284,18 @@ module Mongo
 
     private
 
-    def setup!(remaining)
+    def scheme
+      MONGODB_SCHEME
+    end
+
+    def parse_creds_hosts!(string)
+      hosts, creds = split_creds_hosts(string)
+      @servers = parse_servers!(hosts)
+      @user = parse_user!(creds)
+      @password = parse_password!(creds)
+    end
+
+    def parse!(remaining)
       creds_hosts, db_opts = extract_db_opts!(remaining)
       parse_creds_hosts!(creds_hosts)
       parse_db_opts!(db_opts)
@@ -257,13 +310,6 @@ module Mongo
       [ creds_hosts, db_opts ].map { |s| s.reverse }
     end
 
-    def parse_creds_hosts!(string)
-      hosts, creds = split_creds_hosts(string)
-      @servers = parse_servers!(hosts)
-      @user = parse_user!(creds)
-      @password = parse_password!(creds)
-    end
-
     def split_creds_hosts(string)
       hosts, _, creds = string.reverse.partition(AUTH_DELIM)
       hosts, creds = creds, hosts if hosts.empty?
@@ -293,15 +339,27 @@ module Mongo
 
     def parse_user!(string)
       if (string && user = string.partition(AUTH_USER_PWD_DELIM)[0])
-        raise_invalid_error!(UNESCAPED_USER_PWD) if user =~ UNSAFE
-        decode(user) if user.length > 0
+        if user.length > 0
+          raise_invalid_error!(UNESCAPED_USER_PWD) if user =~ UNSAFE
+          user_decoded = decode(user)
+          if user_decoded =~ PERCENT_CHAR && encode(user_decoded) != user
+            raise_invalid_error!(UNESCAPED_USER_PWD)
+          end
+          user_decoded
+        end
       end
     end
 
     def parse_password!(string)
       if (string && pwd = string.partition(AUTH_USER_PWD_DELIM)[2])
-        raise_invalid_error!(UNESCAPED_USER_PWD) if pwd =~ UNSAFE
-        decode(pwd) if pwd.length > 0
+        if pwd.length > 0
+          raise_invalid_error!(UNESCAPED_USER_PWD) if pwd =~ UNSAFE
+          pwd_decoded = decode(pwd)
+          if pwd_decoded =~ PERCENT_CHAR && encode(pwd_decoded) != pwd
+            raise_invalid_error!(UNESCAPED_USER_PWD)
+          end
+          pwd_decoded
+        end
       end
     end
 
@@ -330,17 +388,22 @@ module Mongo
           validate_port_string!(p)
         elsif host =~ UNIX_SOCKET
           raise_invalid_error!(UNESCAPED_UNIX_SOCKET) if host =~ UNSAFE
+          host = decode(host)
         end
         servers << host
       end
     end
 
     def raise_invalid_error!(details)
-      raise Error::InvalidURI.new(@string, details)
+      raise Error::InvalidURI.new(@string, details, FORMAT)
     end
 
     def decode(value)
-      ::URI::DEFAULT_PARSER.unescape(value)
+      ::URI.decode(value)
+    end
+
+    def encode(value)
+      ::URI.encode(value)
     end
 
     # Hash for storing map of URI option parameters to conversion strategies
@@ -395,6 +458,8 @@ module Mongo
 
     # Client Options
     uri_option 'appname', :app_name
+    uri_option 'compressors', :compressors, :type => :array
+    uri_option 'zlibcompressionlevel', :zlib_compression_level
 
     # Casts option values that do not have a specifically provided
     # transformation to the appropriate type.
@@ -572,5 +637,16 @@ module Mongo
         set.merge(decode(k).downcase.to_sym => decode(v))
       end
     end
+
+    # Extract values from the string and put them into an array.
+    #
+    # @param [ String ] value The string to build an array from.
+    #
+    # @return [ Array ] The array built from the string.
+    def array(value)
+      value.split(',')
+    end
   end
 end
+
+require 'mongo/uri/srv_protocol'
