@@ -18,6 +18,8 @@ describe Mongo::Auth::SCRAM do
     double('cluster').tap do |cl|
       allow(cl).to receive(:topology).and_return(topology)
       allow(cl).to receive(:app_metadata).and_return(app_metadata)
+      allow(cl).to receive(:cluster_time).and_return(nil)
+      allow(cl).to receive(:update_cluster_time)
     end
   end
 
@@ -54,6 +56,16 @@ describe Mongo::Auth::SCRAM do
           cr.login(connection)
         }.to raise_error(Mongo::Auth::Unauthorized)
       end
+
+      context 'when compression is used', if: testing_compression? do
+
+        it 'does not compress the message' do
+          expect(Mongo::Protocol::Compressed).not_to receive(:new)
+          expect {
+            cr.login(connection)
+          }.to raise_error(Mongo::Auth::Unauthorized)
+        end
+      end
     end
   end
 
@@ -67,8 +79,20 @@ describe Mongo::Auth::SCRAM do
       cr.login(connection).documents[0]
     end
 
-    it 'logs the user into the connection', if: list_command_enabled? do
+    after do
+      root_user.instance_variable_set(:@client_key, nil)
+    end
+
+    it 'logs the user into the connection and caches the client key', if: list_command_enabled? do
       expect(login['ok']).to eq(1)
+      expect(root_user.send(:client_key)).not_to be_nil
+    end
+
+    it 'raises an exception when an incorrect client key is set', if: list_command_enabled? do
+      root_user.instance_variable_set(:@client_key, "incorrect client key")
+      expect {
+        cr.login(connection)
+      }.to raise_error(Mongo::Auth::Unauthorized)
     end
   end
 end
