@@ -179,6 +179,24 @@ describe Mongo::Collection::View::Readable do
       it_behaves_like 'a read concern aware operation'
     end
 
+    context 'when a session supporting causal consistency is used' do
+
+      let(:view) do
+        Mongo::Collection::View.new(collection, selector, session: session)
+      end
+
+      let(:operation) do
+        begin; view.map_reduce(map, reduce).to_a; rescue; end
+      end
+
+      let(:command) do
+        operation
+        EventSubscriber.started_events.find { |cmd| cmd.command_name == 'mapreduce' }.command
+      end
+
+      it_behaves_like 'an operation supporting causally consistent reads'
+    end
+
     context 'when not iterating the map/reduce' do
 
       it 'returns the map/reduce object' do
@@ -305,7 +323,7 @@ describe Mongo::Collection::View::Readable do
       end
     end
 
-    it 'takes a read preference option' do
+    it 'takes a read preference option', unless: sharded? do
       expect(view.count(read: { mode: :secondary })).to eq(10)
     end
 
@@ -460,13 +478,13 @@ describe Mongo::Collection::View::Readable do
       end
     end
 
-    it 'takes a max_time_ms option', if: write_command_enabled? do
+    it 'takes a max_time_ms option' do
       expect {
         view.count(max_time_ms: 0.1)
       }.to raise_error(Mongo::Error::OperationFailure)
     end
 
-    it 'sets the max_time_ms option on the command', if: write_command_enabled? do
+    it 'sets the max_time_ms option on the command' do
       expect(view.count(max_time_ms: 100)).to eq(10)
     end
 
@@ -830,7 +848,7 @@ describe Mongo::Collection::View::Readable do
       end
     end
 
-    context 'when a max_time_ms is specified', if: write_command_enabled? do
+    context 'when a max_time_ms is specified' do
 
       let(:documents) do
         (1..3).map{ |i| { field: "test" }}
@@ -960,7 +978,7 @@ describe Mongo::Collection::View::Readable do
       end
 
       it 'does not apply the collation to the distinct' do
-        expect(result).to eq(['bang', 'BANG'])
+        expect(result).to match_array(['bang', 'BANG'])
       end
     end
   end
@@ -1134,28 +1152,19 @@ describe Mongo::Collection::View::Readable do
 
   describe '#read' do
 
-    context 'when providing a hash' do
-
-      it 'converts to a read preference' do
-        expect(view.read(:mode => :primary_preferred).read).to be_a(
-          Mongo::ServerSelector::PrimaryPreferred
-        )
-      end
-    end
-
     context 'when a read pref is specified' do
 
       let(:options) do
-        { :read => Mongo::ServerSelector.get(:mode => :secondary) }
+        { :read => { :mode => :secondary } }
       end
 
       let(:new_read) do
-        Mongo::ServerSelector.get(:mode => :secondary_preferred)
+        { :mode => :secondary_preferred }
       end
 
       it 'sets the read preference' do
         new_view = view.read(new_read)
-        expect(new_view.read).to eq(new_read)
+        expect(new_view.read).to eq(BSON::Document.new(new_read))
       end
 
       it 'returns a new View' do
