@@ -1,6 +1,45 @@
 require 'spec_helper'
 
 describe Mongo::URI do
+
+  describe '.get' do
+
+    let(:uri) { described_class.get(string) }
+
+    context 'when the scheme is mongodb://' do
+
+      let(:string) do
+        'mongodb://localhost:27017'
+      end
+
+      it 'returns a Mongo::URI object' do
+        expect(uri).to be_a(Mongo::URI)
+      end
+    end
+
+    context 'when the scheme is mongodb+srv://', if: test_connecting_externally? do
+
+      let(:string) do
+        'mongodb+srv://test5.test.build.10gen.cc'
+      end
+
+      it 'returns a Mongo::URI::SRVProtocol object' do
+        expect(uri).to be_a(Mongo::URI::SRVProtocol)
+      end
+    end
+
+    context 'when the scheme is invalid' do
+
+      let(:string) do
+        'mongo://localhost:27017'
+      end
+
+      it 'raises an exception' do
+        expect { uri }.to raise_error(Mongo::Error::InvalidURI)
+      end
+    end
+  end
+
   let(:scheme) { 'mongodb://' }
   let(:uri) { described_class.new(string) }
 
@@ -263,7 +302,7 @@ describe Mongo::URI do
       let(:servers) { '%2Ftmp%2Fmongodb-27017.sock' }
 
       it 'returns an array with the parsed server' do
-        expect(uri.servers).to eq([servers])
+        expect(uri.servers).to eq([URI.unescape(servers)])
       end
     end
 
@@ -301,11 +340,11 @@ describe Mongo::URI do
       expect(options[:database]).to eq(TEST_DB)
     end
 
-    it 'includes the credentials in the options' do
+    it 'includes the user in the options' do
       expect(options[:user]).to eq(user)
     end
 
-    it 'includes the options in the options' do
+    it 'includes the password in the options' do
       expect(options[:password]).to eq(password)
     end
   end
@@ -566,7 +605,7 @@ describe Mongo::URI do
 
           it 'raises an exception when read preference is accessed on the client' do
             expect {
-              Mongo::Client.new(string).read_preference
+              Mongo::Client.new(string).server_selector
             }.to raise_exception(Mongo::Error::InvalidServerPreference)
           end
         end
@@ -578,7 +617,7 @@ describe Mongo::URI do
           end
 
           it 'does not raise an exception until the read preference is used' do
-            expect(Mongo::Client.new(string).read_preference).to be_a(Mongo::ServerSelector::Secondary)
+            expect(Mongo::Client.new(string).read_preference).to eq(BSON::Document.new(mode: :secondary, max_staleness: 89))
           end
         end
       end
@@ -929,5 +968,34 @@ describe Mongo::URI do
         expect(Mongo::Client.new(string).options[:app_name]).to eq(:reports)
       end
     end
+
+    context 'when a supported compressors option is provided' do
+      let(:options) { "compressors=zlib" }
+
+      it 'sets the compressors as an array on the client' do
+        expect(Mongo::Client.new(string).options[:compressors]).to eq(['zlib'])
+      end
+    end
+
+    context 'when a non-supported compressors option is provided' do
+      let(:options) { "compressors=snoopy" }
+
+      let(:client) do
+        Mongo::Client.new(string)
+      end
+
+      it 'sets no compressors on the client and warns' do
+        expect(Mongo::Logger.logger).to receive(:warn)
+        expect(client.options[:compressors]).to be_nil
+      end
+    end
+
+    context 'when a zlibCompressionLevel option is provided' do
+      let(:options) { "zlibCompressionLevel=6" }
+
+      it 'sets the zlib compression level on the client' do
+        expect(Mongo::Client.new(string).options[:zlib_compression_level]).to eq(6)
+      end
+    end
   end
 end
