@@ -2,6 +2,17 @@ require 'spec_helper'
 
 describe Mongo::WriteConcern::Acknowledged do
 
+  describe '#acknowledged?' do
+
+    let(:concern) do
+      described_class.new(:w => :majority)
+    end
+
+    it 'returns true' do
+      expect(concern.acknowledged?).to be(true)
+    end
+  end
+
   describe '#get_last_error' do
 
     let(:get_last_error) do
